/*
 * Project name:
     SPI Demo
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Description:
      A simple example of using the SPI library.
 * Test configuration:
     Device:          FT900
                      http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
     Dev.Board:       Clicker 2 for FT900
                      http://www.mikroe.com/ft90x/clicker-2/
     Oscillator:      CPU at full system clock, 100.000MHz
     Ext. Modules:    Thumbstick click - ac:Thumbstick_click
                      UART click       - ac:UART_Click
     SW:              mikroC PRO for FT90x
                      http://www.mikroe.com/mikroc/ft90x/
 * NOTES:
     - Place Thumbstick click in the mikroBUS socket 2
     - Place UART click       in the mikroBUS socket 1
     - Use USART terminal to see the position of the tumbstick which is sent via UART from the MCU
 */

#include <stdint.h>

#define REFRESH_PERIOD 500

#define THR_VAL 5
#define THRESHOLD_CHECK(a)          ( ((a) < (-THR_VAL)) || ((a) > (THR_VAL)) )
#define THRESHOLD_DOUBLE_CHECK(a,b) ( THRESHOLD_CHECK(a) || THRESHOLD_CHECK(b) )

typedef enum LineEnding
{
  WIN,              // CRLF for windows
  UNIX              // LF for unix
} LineEnding;

typedef enum ButtonStates
{
  unknown,          // unknown state
  pressed,          // button pressed
  released          // button released
} ButtonStates;

sbit
  ADC_Chip_Select at GPIO_PIN33_bit;

uint16_t
  baudRate = 56000, // desired baud rate
  horVal   = 0,     // horisontal value
  verVal   = 0,     // vertical   value
  oldHV    = 0,     // previous horisontal value
  oldVV    = 0,     // previous vertical   value
  midHV    = 0,     // horisontal value while the thumbstick is still
  midVV    = 0;     // vertical   value while the thumbstick is still

uint8_t
  tempStr[256];     // buffer for writing values

ButtonStates
  state    = unknown, // state of the button
  oldState = unknown;

LineEnding
  linEnd   = WIN;   // desired line ending

/**
 * Sends the line along with its terminating characters
 * @param text     pointer to string to be sent
 * @param linEnd   choose the line ending either for WIN or UNIX
 */
void UART1_Write_Line(uint8_t* text,
                    LineEnding linEnd)
{
  if (linEnd == WIN)
  {
    UART1_Write_Text(text);
    UART1_Write(13);
    UART1_Write(10);
  }
  else
  {
    UART1_Write_Text(text);
    UART1_Write(10);
  }
}

/**
 * Initialize UART and SPI interface
 */
void Init_Periph()
{
  // Initialize UART and send START msg
  UART1_Init      (baudRate);
  UART1_Write_Line("ADC Demo", linEnd);
  UART1_Write_Line("Start",    linEnd);

  // Initialize SPI interface
  SPIM1_Init();

  // Set Chip_Select pin as output
  GPIO_Digital_Output(&GPIO_PORT_32_39, _GPIO_PINMASK_1);
  ADC_Chip_Select = 1;

  // Set IN pin to as input
  GPIO_Digital_Input(&GPIO_PORT_00_07,  _GPIO_PINMASK_4);
}

/**
 * Reads ADC value via SPI from MCP3204
 * @param  adcChannel desired ADC channel
 * @return            ADC value
 */
uint16_t ReadADC(uint8_t adcChannel)
{
  uint16_t
    adcValue;

  // Select MCP3204
  ADC_Chip_Select = 0;

  // Bits 7 & 6 define ADC input
  adcChannel <<= 6;

  // SPI communication using 8-bit segments
  SPIM1_Write(0x06);

  adcValue   = SPIM1_Read(adcChannel) & 0x0F; // Get higher byte of ADC Value and keep the first four
  adcValue <<= 8;                             // shift it up

  // Get the lower byte of ADC value
  adcValue = adcValue | SPIM1_Read(0);

  // Deselect MCP3204
  ADC_Chip_Select = 1;

  return adcValue;
}

/**
 * Displays current thumbstick position via UART
 */
void DisplayValues()
{
  int16_t
    tmpValH = 0,
    tmpValV = 0;

  tmpValH = horVal - oldHV;
  tmpValV = verVal - oldVV;

  if ( THRESHOLD_DOUBLE_CHECK(tmpValH,tmpValV) )
  {
    oldHV = horVal;
    oldVV = verVal;

    UART1_Write_Text("Positions: ");

    tmpValH = horVal - midHV;
    if ( THRESHOLD_CHECK(tmpValH) )
    {
      if (tmpValH > 0)      // the thumbstick is pressed to the right
      {
        WordToStr       ((uint16_t)tmpValH, tempStr);
        UART1_Write_Text("right: ");
      }
      else if (tmpValH < 0) // the thumbstick is pressed to the left
      {
        WordToStr       ((uint16_t)(-tmpValH), tempStr);
        UART1_Write_Text("left: ");
      }
      UART1_Write_Text(tempStr);
    }
    else
    {
      UART1_Write_Text("hor. middle");
    }

    UART1_Write_Text(" & ");

    tmpValV = verVal - midVV;
    if ( THRESHOLD_CHECK(tmpValV) )
    {
      if (tmpValV > 0)      // the thumbstick is pressed down
      {
        WordToStr       ((uint16_t)tmpValV, tempStr);
        UART1_Write_Text("down: ");
      }
      else if (tmpValV < 0) // the thumbstick is pressed up
      {
        WordToStr       ((uint16_t)(-tmpValV), tempStr);
        UART1_Write_Text("up: ");
      }
      UART1_Write_Line(tempStr, linEnd);
    }
    else
    {
      UART1_Write_Line("ver. middle", linEnd);
    }
  }
}

/**
 *  loop, read the current values from ADC via SPI and write them via UART
 */
void main()
{
  Init_Periph();

  // Set middle values
  midVV = ReadADC(0);
  midHV = ReadADC(1);

  while (1)
  {
    verVal  = ReadADC(0);
    horVal  = ReadADC(1);

    if (GPIO_PIN4_bit == 0)
    {
      state = pressed;
    }
    else if (GPIO_PIN4_bit == 1)
    {
      state = released;
    }

    switch(state)
    {
      case pressed:
        if (oldState == released)
        {
          UART1_Write_Line("Button pressed", linEnd);
        }
        oldState = state;
        state = unknown;
        break;
      case released:
        if (oldState == pressed)
        {
          UART1_Write_Line("Button released", linEnd);
        }
        else
        {
          DisplayValues();
        }
        oldState = state;
        state = unknown;
        break;
    }
    Delay_ms(REFRESH_PERIOD);
  }
}
